import json
import os
import requests


RESOURCE_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), '.resources')


def resource_filepath(filename):
  """Returns a path to the given file name in the test resources directory."""
  return os.path.join(RESOURCE_PATH, filename)


def prepare_test_resources():
  """Prepares any resources that are expected to be available at test-time."""

  # Generate the Dialpad API swagger spec, and write it to a file for easy access.
  with open(resource_filepath('swagger_spec.json'), 'w') as f:
    json.dump(_generate_swagger_spec(), f)


def _generate_swagger_spec():
  """Downloads current Dialpad API swagger spec and returns it as a dict."""

  # Unfortunately, a little bit of massaging is needed to appease the swagger parser.
  def _hotpatch_spec_piece(piece):
    if 'type' in piece:
      if piece['type'] == 'string' and piece.get('format') == 'int64' and 'default' in piece:
        piece['default'] = str(piece['default'])

      if 'operationId' in piece and 'parameters' in piece:
        for sub_p in piece['parameters']:
          sub_p['required'] = sub_p.get('required', False)

    if 'basePath' in piece:
      del piece['basePath']

  def _hotpatch_spec(spec):
    if isinstance(spec, dict):
      _hotpatch_spec_piece(spec)
      for k, v in spec.iteritems():
        _hotpatch_spec(v)

    elif isinstance(spec, list):
      for v in spec:
        _hotpatch_spec(v)

    return spec
  
  # Download the spec from dialpad.com.
  spec_json = requests.get('https://dialpad.com/static/openapi/apiv2openapi-en.json').json()

  # Return a patched version that will satisfy the swagger lib.
  return _hotpatch_spec(spec_json)



from .resource import DialpadResource

class TranscriptResource(DialpadResource):
  """TranscriptResource implements python bindings for the Dialpad API's transcript endpoints.
  See https://developers.dialpad.com/reference#transcripts for additional documentation.
  """
  _resource_path = ['transcripts']

  def get(self, call_id):
    """Get the transcript of a call.

    Args:
      call_id (int, required): The ID of the call.

    See Also:
      https://developers.dialpad.com/reference#transcriptapi_gettranscript
    """
    return self.request([call_id])



from .resource import DialpadResource

class CallCenterResource(DialpadResource):
  """CallCenterResource implements python bindings for the Dialpad API's call center endpoints.
  See https://developers.dialpad.com/reference#callcenters for additional documentation.
  """

  _resource_path = ['callcenters']

  def get(self, call_center_id):
    """Gets a call center by ID.

    Args:
      call_center_id (int, required): The ID of the call center to retrieve.

    See Also:
      https://developers.dialpad.com/reference#callcenterapi_getcallcenter
    """
    return self.request([call_center_id], method='GET')

  def get_operators(self, call_center_id):
    """Gets the list of users who are operators for the specified call center.

    Args:
      call_center_id (int, required): The ID of the call center.

    See Also:
      https://developers.dialpad.com/reference#callcenterapi_listoperators
    """
    return self.request([call_center_id, 'operators'], method='GET')

  def add_operator(self, call_center_id, user_id, **kwargs):
    """Adds the specified user as an operator of the specified call center.

    Args:
      call_center_id (int, required): The ID of the call center.
      user_id (int, required): The ID of the user to add as an operator.
      skill_level (int, optional): Skill level of the operator. Integer value in range 1 - 100.
                                   Default 100
      role (str, optional): The role of the new operator ('operator', 'supervisor', or 'admin').
                            Default 'operator'
      license_type (str, optional): The type of license to assign to the new operator if a license
                                    is required ('agents', or 'lite_support_agents').
                                    Default 'agents'
      keep_paid_numbers (bool, optional): If the operator is currently on a license that provides
                                          paid numbers and `license_type` is set to
                                          `lite_support_agents`, this option will determine if the
                                          operator keeps those numbers. Set to False for the
                                          numbers to be removed.
                                          Default True

    See Also:
      https://developers.dialpad.com/reference#callcenterapi_addoperator
    """
    kwargs['user_id'] = user_id
    return self.request([call_center_id, 'operators'], method='POST', data=kwargs)

  def remove_operator(self, call_center_id, user_id):
    """Removes the specified user from the specified call center.

    Args:
      call_center_id (int, required): The ID of the call center.
      user_id (int, required): The ID of the user to remove.

    See Also:
      https://developers.dialpad.com/reference#callcenterapi_removeoperator
    """
    return self.request([call_center_id, 'operators'], method='DELETE', data={'user_id': user_id})



from .resource import DialpadResource

class CompanyResource(DialpadResource):
  """CompanyResource implements python bindings for the Dialpad API's company endpoints.
  
  See https://developers.dialpad.com/reference#company for additional documentation.
  """
  _resource_path = ['company']

  def get(self):
    """Gets the company resource.

    See Also:
      https://developers.dialpad.com/reference#companyapi_getcompany
    """
    return self.request(method='GET')



from .resource import DialpadResource

class OfficeResource(DialpadResource):
  """OfficeResource implements python bindings for the Dialpad API's office endpoints.
  See https://developers.dialpad.com/reference#offices for additional documentation.
  """

  _resource_path = ['offices']

  def list(self, limit=25, **kwargs):
    """Lists the company's offices.

    Args:
      limit (int, optional): the number of offices to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#officeapi_listoffices
    """
    return self.request(method='GET', data=dict(limit=limit, **kwargs))

  def get(self, office_id):
    """Gets an office by ID.

    Args:
      office_id (int, required): The ID of the office to retrieve.

    See Also:
      https://developers.dialpad.com/reference#officeapi_getoffice
    """
    return self.request([office_id], method='GET')

  def assign_number(self, office_id, **kwargs):
    """Assigns a phone number to the specified office

    Args:
      office_id (int, required): The ID of the office to which the number should be assigned.
      number (str, optional): An e164 number that has already been allocated to the company's
                              reserved number pool that should be re-assigned to this office.
      area_code (str, optional): The area code to use to filter the set of available numbers to be
                                 assigned to this office.

    See Also:
      https://developers.dialpad.com/reference#numberapi_assignnumbertooffice
    """
    return self.request([office_id, 'assign_number'], method='POST', data=kwargs)

  def get_operators(self, office_id):
    """Gets the list of users who are operators for the specified office.

    Args:
      office_id (int, required): The ID of the office.

    See Also:
      https://developers.dialpad.com/reference#officeapi_listoperators
    """
    return self.request([office_id, 'operators'], method='GET')

  def unassign_number(self, office_id, number):
    """Unassigns the specified number from the specified office.

    Args:
      office_id (int, required): The ID of the office.
      number (str, required): The e164-formatted number that should be unassigned from the office.

    See Also:
      https://developers.dialpad.com/reference#numberapi_unassignnumberfromoffice
    """
    return self.request([office_id, 'unassign_number'], method='POST', data={'number': number})

  def get_call_centers(self, office_id, limit=25, **kwargs):
    """Lists the call centers under the specified office.

    Args:
      office_id (int, required): The ID of the office.
      limit (int, optional): the number of call centers to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#callcenterapi_listcallcenters
    """
    return self.request([office_id, 'callcenters'], method='GET', data=dict(limit=limit, **kwargs))

  def get_departments(self, office_id, limit=25, **kwargs):
    """Lists the departments under the specified office.

    Args:
      office_id (int, required): The ID of the office.
      limit (int, optional): the number of departments to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#departmentapi_listdepartments
    """
    return self.request([office_id, 'departments'], method='GET', data=dict(limit=limit, **kwargs))

  def get_plan(self, office_id):
    """Gets the plan associated with the office.

    (i.e. a breakdown of the licenses that have been purchased for the specified office)

    Args:
      office_id (int, required): The ID of the office.

    See Also:
      https://developers.dialpad.com/reference#planapi_getplan
    """
    return self.request([office_id, 'plan'], method='GET')

  def update_licenses(self, office_id, fax_line_delta):
    """Updates the number of licenses in the specified office's plan.

    Note:
      Currently only supports modifying the number of fax line licenses.

    Args:
      office_id (int, required): The ID of the office.
      fax_line_delta (int, required): Number of fax lines to add. If a negative number is specified,
                                      then fax lines will be removed rather than added.

    See Also:
      https://developers.dialpad.com/reference#planapi_updatelicenses
    """
    return self.request([office_id, 'plan', 'updateLicenses'], method='POST', data={
        'fax_line_delta': fax_line_delta
    })



from .resource import DialpadResource

class SMSResource(DialpadResource):
  """SMSResource implements python bindings for the Dialpad API's sms endpoints.
  See https://developers.dialpad.com/reference#sms for additional documentation.
  """
  _resource_path = ['sms']

  def send_sms(self, user_id, to_numbers, text, **kwargs):
    """Sends an SMS message on behalf of the specified user.

    Args:
      user_id (int, required): The ID of the user that should be sending the SMS.
      to_numbers (list<str>, required): A list of one-or-more e164-formatted phone numbers which
                                        should receive the SMS.
      text (str, required): The content of the SMS message.
      infer_country_code (bool, optional): If set, the e164-contraint will be relaxed on to_numbers,
                                           and potentially ambiguous numbers will be assumed to be
                                           numbers in the specified user's country.
      sender_group_id (int, optional): The ID of an office, department, or call center that the user
                                       should send the SMS on behalf of.
      sender_group_type (str, optional): The ID type (i.e. office, department, or callcenter).

    See Also:
      https://developers.dialpad.com/reference#roomapi_listrooms
    """
    return self.request(method='POST', data=dict(text=text, user_id=user_id, to_numbers=to_numbers,
                                                 **kwargs))



from .resource import DialpadResource

class RoomResource(DialpadResource):
  """RoomResource implements python bindings for the Dialpad API's room endpoints.
  See https://developers.dialpad.com/reference#rooms for additional documentation.
  """

  _resource_path = ['rooms']

  def list(self, limit=25, **kwargs):
    """Lists rooms in the company.

    Args:
      limit (int, optional): The number of rooms to fetch per request.
      office_id (int, optional): If specified, only rooms associated with that office will be
                                 returned.

    See Also:
      https://developers.dialpad.com/reference#roomapi_listrooms
    """
    return self.request(method='GET', data=dict(limit=limit, **kwargs))

  def create(self, name, office_id):
    """Creates a new room with the specified name within the specified office.

    Args:
      name (str, required): A human-readable name for the room.
      office_id (int, required): The ID of the office.

    See Also:
      https://developers.dialpad.com/reference#roomapi_createroom
    """
    return self.request(method='POST', data={'name': name, 'office_id': office_id})

  def generate_international_pin(self, customer_ref):
    """Creates a PIN to allow an international call to be made from a room phone.

    Args:
      customer_ref (str, required): An identifier to be printed in the usage summary. Typically used
                                    for identifying the person who requested the PIN

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_createinternationalpin
    """
    return self.request(['international_pin'], method='POST', data={'customer_ref': customer_ref})

  def delete(self, room_id):
    """Deletes a room by ID.

    Args:
      room_id (str, required): The ID of the room to be deleted.

    See Also:
      https://developers.dialpad.com/reference#roomapi_deleteroom
    """
    return self.request([room_id], method='DELETE')

  def get(self, room_id):
    """Gets a room by ID.

    Args:
      room_id (str, required): The ID of the room to be fetched.

    See Also:
      https://developers.dialpad.com/reference#roomapi_getroom
    """
    return self.request([room_id], method='GET')

  def update(self, room_id, **kwargs):
    """Updates the specified room.

    Args:
      room_id (str, required): The ID of the room to be updated.
      name (str, optional): A human-readable name for the room.
      phone_numbers (list<str>, optional): The list of e164-formatted phone numbers that should be
                                           associated with this room. New numbers will be assigned,
                                           and omitted numbers will be unassigned.

    See Also:
      https://developers.dialpad.com/reference#roomapi_updateroom
    """
    return self.request([room_id], method='PATCH', data=kwargs)

  def assign_number(self, room_id, **kwargs):
    """Assigns a phone number to the specified room

    Args:
      room_id (int, required): The ID of the room to which the number should be assigned.
      number (str, optional): An e164 number that has already been allocated to the company's
                              reserved number pool that should be re-assigned to this office.
      area_code (str, optional): The area code to use to filter the set of available numbers to be
                                 assigned to this office.

    See Also:
      https://developers.dialpad.com/reference#numberapi_assignnumbertoroom
    """
    return self.request([room_id, 'assign_number'], method='POST', data=kwargs)

  def unassign_number(self, room_id, number):
    """Unassigns the specified number from the specified room.

    Args:
      room_id (int, required): The ID of the room.
      number (str, required): The e164-formatted number that should be unassigned from the room.

    See Also:
      https://developers.dialpad.com/reference#numberapi_unassignnumberfromroom
    """
    return self.request([room_id, 'unassign_number'], method='POST', data={'number': number})

  def get_deskphones(self, room_id):
    """Lists the phones that are assigned to the specified room.

    Args:
      room_id (int, required): The ID of the room.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_listroomdeskphones
    """
    return self.request([room_id, 'deskphones'], method='GET')

  def create_deskphone(self, room_id, mac_address, name, phone_type):
    """Creates a desk phone belonging to the specified room.

    Args:
      room_id (int, required): The ID of the room.
      mac_address (str, required): MAC address of the desk phone.
      name (str, required): A human-readable name for the desk phone.
      phone_type (str, required): Type (vendor) of the desk phone. One of "obi", "polycom", "sip",
                                  "mini", "audiocodes", "yealink". Use "sip" for generic types.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_createroomdeskphone
    """
    return self.request([room_id, 'deskphones'], method='POST', data={
      'mac_address': mac_address,
      'name': name,
      'type': phone_type,
    })

  def delete_deskphone(self, room_id, deskphone_id):
    """Deletes the specified desk phone.

    Args:
      room_id (int, required): The ID of the room.
      deskphone_id (str, required): The ID of the desk phone.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_deleteroomdeskphone
    """
    return self.request([room_id, 'deskphones', deskphone_id], method='DELETE')

  def get_deskphone(self, room_id, deskphone_id):
    """Gets the specified desk phone.

    Args:
      room_id (int, required): The ID of the room.
      deskphone_id (str, required): The ID of the desk phone.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_getroomdeskphone
    """
    return self.request([room_id, 'deskphones', deskphone_id], method='GET')



from .resource import DialpadResource

class EventSubscriptionResource(DialpadResource):
  """EventSubscriptionResource implements python bindings for the Dialpad API's event subscription
  endpoints.
  
  See https://developers.dialpad.com/reference#event for additional documentation.
  """
  _resource_path = ['event-subscriptions']

  def list_call_event_subscriptions(self, limit=25, **kwargs):
    """Lists call event subscriptions.

    Args:
      limit (int, optional): The number of subscriptions to fetch per request
      target_id (str, optional): The ID of a specific target to use as a filter
      target_type (str, optional): The type of the target (one of "department", "office",
                                   "callcenter", "user", "room", "staffgroup", "callrouter",
                                   "channel", "coachinggroup", or "unknown")

    See Also:
      https://developers.dialpad.com/reference#calleventsubscriptionapi_listcalleventsubscriptions
    """
    return self.request(['call'], method='GET', data=dict(limit=limit, **kwargs))

  def get_call_event_subscription(self, subscription_id):
    """Gets a specific call event subscription.

    Args:
      subscription_id (str, required): The ID of the subscription

    See Also:
      https://developers.dialpad.com/reference#calleventsubscriptionapi_getcalleventsubscription
    """
    return self.request(['call', subscription_id], method='GET')

  def put_call_event_subscription(self, subscription_id, url, enabled=True, group_calls_only=False,
                                  **kwargs):
    """Update or create a call event subscription.

    The subscription_id is required. If the ID exists, then this call will update the subscription
    resource. If the ID does not exist, then it will create a new subscription with that ID.

    Args:
      subscription_id (str, required): The ID of the subscription
      url (str, required): The URL which should be called when the subscription fires
      secret (str, optional): A secret to use to encrypt subscription event payloads
      enabled (bool, optional): Whether or not the subscription should actually fire
      group_calls_only (bool, optional): Whether to limit the subscription to only fire if the call
                                         is a group call
      target_id (str, optional): The ID of a specific target to use as a filter
      target_type (str, optional): The type of the target (one of "department", "office",
                                   "callcenter", "user", "room", "staffgroup", "callrouter",
                                   "channel", "coachinggroup", or "unknown")
      call_states (list<str>, optional): The specific types of call events that should trigger the
                                         subscription (any of "preanswer", "calling", "ringing",
                                         "connected", "merged", "hold", "queued", "voicemail",
                                         "eavesdrop", "monitor", "barge", "hangup", "blocked",
                                         "admin", "parked", "takeover", "all", "postcall",
                                         "transcription", or "recording")

    See Also:
      https://developers.dialpad.com/reference#calleventsubscriptionapi_createorupdatecalleventsubscription
    """

    return self.request(['call', subscription_id], method='PUT',
                        data=dict(url=url, enabled=enabled, group_calls_only=group_calls_only,
                                  **kwargs))

  def delete_call_event_subscription(self, subscription_id):
    """Deletes a specific call event subscription.

    Args:
      subscription_id (str, required): The ID of the subscription

    See Also:
      https://developers.dialpad.com/reference#calleventsubscriptionapi_deletecalleventsubscription
    """
    return self.request(['call', subscription_id], method='DELETE')


  def list_sms_event_subscriptions(self, limit=25, **kwargs):
    """Lists sms event subscriptions.

    Args:
      limit (int, optional): The number of subscriptions to fetch per request
      target_id (str, optional): The ID of a specific target to use as a filter
      target_type (str, optional): The type of the target (one of "department", "office",
                                   "callcenter", "user", "room", "staffgroup", "callrouter",
                                   "channel", "coachinggroup", or "unknown")

    See Also:
      https://developers.dialpad.com/reference#smseventsubscriptionapi_listsmseventsubscriptions
    """
    return self.request(['sms'], method='GET', data=dict(limit=limit, **kwargs))

  def get_sms_event_subscription(self, subscription_id):
    """Gets a specific sms event subscription.

    Args:
      subscription_id (str, required): The ID of the subscription

    See Also:
      https://developers.dialpad.com/reference#smseventsubscriptionapi_getsmseventsubscription
    """
    return self.request(['sms', subscription_id], method='GET')

  def put_sms_event_subscription(self, subscription_id, url, direction, enabled=True,
                                  **kwargs):
    """Update or create an sms event subscription.

    The subscription_id is required. If the ID exists, then this call will update the subscription
    resource. If the ID does not exist, then it will create a new subscription with that ID.

    Args:
      subscription_id (str, required): The ID of the subscription
      url (str, required): The URL which should be called when the subscription fires
      direction (str, required): The SMS direction that should fire the subscripion ("inbound",
                                 "outbound", or "all")
      enabled (bool, optional): Whether or not the subscription should actually fire
      secret (str, optional): A secret to use to encrypt subscription event payloads
      target_id (str, optional): The ID of a specific target to use as a filter
      target_type (str, optional): The type of the target (one of "department", "office",
                                   "callcenter", "user", "room", "staffgroup", "callrouter",
                                   "channel", "coachinggroup", or "unknown")

    See Also:
      https://developers.dialpad.com/reference#smseventsubscriptionapi_createorupdatesmseventsubscription
    """

    return self.request(['sms', subscription_id], method='PUT',
                        data=dict(url=url, enabled=enabled, direction=direction,
                                  **kwargs))

  def delete_sms_event_subscription(self, subscription_id):
    """Deletes a specific sms event subscription.

    Args:
      subscription_id (str, required): The ID of the subscription

    See Also:
      https://developers.dialpad.com/reference#smseventsubscriptionapi_deletesmseventsubscription
    """
    return self.request(['sms', subscription_id], method='DELETE')




class DialpadResource(object):
  _resource_path = None

  def __init__(self, client, basepath=None):
    self._client = client

  def request(self, path=None, *args, **kwargs):
    if self._resource_path is None:
      raise NotImplementedError('DialpadResource subclasses must have a _resource_path property')
    path = path or []
    return self._client.request(self._resource_path + path, *args, **kwargs)



from .resource import DialpadResource

class CallbackResource(DialpadResource):
  """CallbackResource implements python bindings for the Dialpad API's callback endpoints.
  
  See https://developers.dialpad.com/reference#callback for additional documentation.
  """
  _resource_path = ['callback']

  def enqueue_callback(self, call_center_id, phone_number):
    """Requests a call-back for the specified number by adding it to the callback queue for the
    specified call center.

    The call back is added to the queue for the call center like a regular call, and a call is
    initiated when the next operator becomes available. This API respects all existing call center
    settings, e.g. business / holiday hours and queue settings. This API currently does not allow
    international call backs. Duplicate call backs for a given number and call center are not
    allowed.

    Args:
      call_center_id (str, required): The ID of the call center for which the callback should be
                                      enqueued.
      phone_number (str, required): The e164-formatted number that should be added to the callback
                                    queue.

    See Also:
      https://developers.dialpad.com/reference#callapi_callback
    """
    return self.request(method='POST', data=dict(call_center_id=call_center_id,
                                                 phone_number=phone_number))



from .resource import DialpadResource

class DepartmentResource(DialpadResource):
  """DepartmentResource implements python bindings for the Dialpad API's department endpoints.
  See https://developers.dialpad.com/reference#departments for additional documentation.
  """

  _resource_path = ['departments']

  def get(self, department_id):
    """Gets a department by ID.

    Args:
      department_id (int, required): The ID of the department to retrieve.

    See Also:
      https://developers.dialpad.com/reference#departmentapi_getdepartment
    """
    return self.request([department_id], method='GET')

  def get_operators(self, department_id):
    """Gets the list of users who are operators for the specified department.

    Args:
      department_id (int, required): The ID of the department.

    See Also:
      https://developers.dialpad.com/reference#departmentapi_listoperators
    """
    return self.request([department_id, 'operators'], method='GET')

  def add_operator(self, department_id, operator_id, operator_type, role='operator'):
    """Adds the specified user as an operator of the specified department.

    Args:
      department_id (int, required): The ID of the department.
      operator_id (int, required): The ID of the operator to add.
      operator_type (str, required): Type of the operator to add ('user' or 'room').
      role (str, optional): The role of the new operator ('operator' or 'admin').
                            Default 'operator'

    See Also:
      https://developers.dialpad.com/reference#departmentapi_addoperator
    """
    return self.request([department_id, 'operators'], method='POST', data={
      'operator_id': operator_id,
      'operator_type': operator_type,
      'role': role,
    })

  def remove_operator(self, department_id, operator_id, operator_type):
    """Removes the specified user from the specified department.

    Args:
      department_id (int, required): The ID of the department.
      operator_id (int, required): The ID of the operator to remove.
      operator_type (str, required): Type of the operator to remove ('user' or 'room').

    See Also:
      https://developers.dialpad.com/reference#departmentapi_removeoperator
    """
    return self.request([department_id, 'operators'], method='DELETE', data={
      'operator_id': operator_id,
      'operator_type': operator_type,
    })



import os
import requests

from cached_property import cached_property

from .resources import BlockedNumberResource
from .resources import CallResource
from .resources import CallbackResource
from .resources import CallCenterResource
from .resources import CompanyResource
from .resources import ContactResource
from .resources import DepartmentResource
from .resources import EventSubscriptionResource
from .resources import NumberResource
from .resources import OfficeResource
from .resources import RoomResource
from .resources import SMSResource
from .resources import StatsExportResource
from .resources import TranscriptResource
from .resources import UserResource
from .resources import UserDeviceResource


class DialpadClient(object):
  def __init__(self, token, sandbox=False, base_url=None):
    self._token = token
    if base_url is not None:
      self._base_url = base_url
    else:
      self._base_url = 'https://sandbox.dialpad.com' if sandbox else 'https://dialpad.com'

  def _url(self, *path):
    path = ['%s' % p for p in path]
    return os.path.join(self._base_url, 'api', 'v2', *path)

  def _cursor_iterator(self, response_json, path, method, data, headers):
    for i in response_json['items']:
      yield i

    data = dict(data or {})

    while 'cursor' in response_json:
      data['cursor'] = response_json['cursor']
      response = self._raw_request(path, method, data, headers)
      response.raise_for_status()
      response_json = response.json()
      for i in response_json['items']:
        yield i

  def _raw_request(self, path, method='GET', data=None, headers=None):
    url = self._url(*path)
    headers = headers or {}
    headers['Authorization'] = 'Bearer %s' % self._token
    if method == 'GET':
      return requests.get(url, params=data, headers=headers)

    if method == 'POST':
      return requests.post(url, json=data, headers=headers)

    if method == 'PUT':
      return requests.put(url, json=data, headers=headers)

    if method == 'PATCH':
      return requests.patch(url, json=data, headers=headers)

    if method == 'DELETE':
      return requests.delete(url, json=data, headers=headers)

    raise ValueError('Unsupported method "%s"' % method)

  def request(self, path, method='GET', data=None, headers=None):
    response = self._raw_request(path, method, data, headers)
    response.raise_for_status()

    response_json = response.json()
    response_keys = set(k for k in response_json)
    # If the response contains the 'items' key, (and maybe 'cursor'), then this is a cursorized
    # list response.
    if 'items' in response_keys and not response_keys - {'cursor', 'items'}:
      return self._cursor_iterator(response_json, path=path, method=method, data=data, headers=headers)
    return response_json

  @cached_property
  def blocked_number(self):
    return BlockedNumberResource(self)

  @cached_property
  def call(self):
    return CallResource(self)

  @cached_property
  def callback(self):
    return CallbackResource(self)

  @cached_property
  def callcenter(self):
    return CallCenterResource(self)

  @cached_property
  def company(self):
    return CompanyResource(self)

  @cached_property
  def contact(self):
    return ContactResource(self)

  @cached_property
  def department(self):
    return DepartmentResource(self)

  @cached_property
  def event_subscription(self):
    return EventSubscriptionResource(self)

  @cached_property
  def number(self):
    return NumberResource(self)

  @cached_property
  def office(self):
    return OfficeResource(self)

  @cached_property
  def room(self):
    return RoomResource(self)

  @cached_property
  def sms(self):
    return SMSResource(self)

  @cached_property
  def stats(self):
    return StatsExportResource(self)

  @cached_property
  def transcript(self):
    return TranscriptResource(self)

  @cached_property
  def user(self):
    return UserResource(self)

  @cached_property
  def userdevice(self):
    return UserDeviceResource(self)



from .resource import DialpadResource

class ContactResource(DialpadResource):
  """ContactResource implements python bindings for the Dialpad API's contact endpoints.
  
  See https://developers.dialpad.com/reference#contacts for additional documentation.
  """
  _resource_path = ['contacts']

  def list(self, limit=25, **kwargs):
    """Lists contacts in the company.

    Args:
      limit (int, optional): The number of contacts to fetch per request.
      owner_id (int, optional): A specific user who's contacts should be listed.

    See Also:
      https://developers.dialpad.com/reference#contactapi_listcontacts
    """
    return self.request(method='GET', data=dict(limit=limit, **kwargs))

  def create(self, first_name, last_name, **kwargs):
    """Creates a new contact.

    Args:
      first_name (str, required): The contact's first name.
      last_name (str, required): The contact's family name.
      company_name (str, optional): The name of the contact's company.
      emails (list<str>, optional): A list of email addresses associated with the contact.
      extension (str, optional): The contact's extension number.
      job_title (str, optional): The contact's job title.
      owner_id (str, optional): The ID of the user who should own this contact. If no owner_id is
                                specified, then a company-level shared contact will be created.
      phones (list<str>, optional): A list of e164 numbers that belong to this contact.
      trunk_group (str, optional): The contact's trunk group.
      urls (list<str>, optional): A list of urls that pertain to this contact.

    See Also:
      https://developers.dialpad.com/reference#contactapi_createcontact
    """
    return self.request(method='POST', data=dict(first_name=first_name, last_name=last_name,
                                                 **kwargs))

  def create_with_uid(self, first_name, last_name, uid, **kwargs):
    """Creates a new contact with a prescribed unique identifier.

    Args:
      first_name (str, required): The contact's first name.
      last_name (str, required): The contact's family name.
      uid (str, required): A unique identifier that should be included in the contact's ID.
      company_name (str, optional): The name of the contact's company.
      emails (list<str>, optional): A list of email addresses associated with the contact.
      extension (str, optional): The contact's extension number.
      job_title (str, optional): The contact's job title.
      phones (list<str>, optional): A list of e164 numbers that belong to this contact.
      trunk_group (str, optional): The contact's trunk group.
      urls (list<str>, optional): A list of urls that pertain to this contact.

    See Also:
      https://developers.dialpad.com/reference#contactapi_createcontactwithuid
    """
    return self.request(method='PUT', data=dict(first_name=first_name, last_name=last_name, uid=uid,
                                                **kwargs))

  def delete(self, contact_id):
    """Deletes the specified contact.

    Args:
      contact_id (str, required): The ID of the contact to delete.

    See Also:
      https://developers.dialpad.com/reference#contactapi_deletecontact
    """
    return self.request([contact_id], method='DELETE')

  def get(self, contact_id):
    """Gets a contact by ID.

    Args:
      contact_id (str, required): The ID of the contact.

    See Also:
      https://developers.dialpad.com/reference#contactapi_getcontact
    """
    return self.request([contact_id], method='GET')

  def patch(self, contact_id, **kwargs):
    """Updates an existing contact.

    Args:
      contact_id (str, required): The ID of the contact.
      first_name (str, optional): The contact's first name.
      last_name (str, optional): The contact's family name.
      company_name (str, optional): The name of the contact's company.
      emails (list<str>, optional): A list of email addresses associated with the contact.
      extension (str, optional): The contact's extension number.
      job_title (str, optional): The contact's job title.
      phones (list<str>, optional): A list of e164 numbers that belong to this contact.
      trunk_group (str, optional): The contact's trunk group.
      urls (list<str>, optional): A list of urls that pertain to this contact.

    See Also:
      https://developers.dialpad.com/reference#contactapi_updatecontact
    """
    return self.request([contact_id], method='PATCH', data=kwargs)



from .resource import DialpadResource

class BlockedNumberResource(DialpadResource):
  """BlockedNumberResource implements python bindings for the Dialpad API's blocked-number
  endpoints.
  
  See https://developers.dialpad.com/reference#blockednumbers for additional documentation.
  """
  _resource_path = ['blockednumbers']

  def list(self, limit=25, **kwargs):
    """List all numbers that have been flagged as "blocked" via the API.

    Args:
      limit (int, optional): The number of numbers to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#blockednumberapi_listnumbers
    """
    return self.request(method='GET', data=dict(limit=limit, **kwargs))

  def block_numbers(self, numbers):
    """Blocks inbound calls from the specified numbers.

    Args:
      numbers (list<str>, required): A list of e164-formatted numbers to block.

    See Also:
      https://developers.dialpad.com/reference#blockednumberapi_addnumbers
    """
    return self.request(['add'], method='POST', data={'numbers': numbers})

  def unblock_numbers(self, numbers):
    """Unblocks inbound calls from the specified numbers.

    Args:
      numbers (list<str>, required): A list of e164-formatted numbers to unblock.

    See Also:
      https://developers.dialpad.com/reference#blockednumberapi_removenumbers
    """
    return self.request(['remove'], method='POST', data={'numbers': numbers})

  def get(self, number):
    """Gets a number object, provided it has been blocked by the API.
    
    Note:
      This API call will 404 if the number is not blocked, and return {"number": <number>} if the
      number is blocked.

    Args:
      number (str, required): An e164-formatted number.

    See Also:
      https://developers.dialpad.com/reference#blockednumberapi_getnumber
    """
    return self.request([number], method='GET')



from .blocked_number import BlockedNumberResource
from .call import CallResource
from .callback import CallbackResource
from .callcenter import CallCenterResource
from .company import CompanyResource
from .contact import ContactResource
from .department import DepartmentResource
from .event_subscription import EventSubscriptionResource
from .number import NumberResource
from .office import OfficeResource
from .room import RoomResource
from .sms import SMSResource
from .stats import StatsExportResource
from .transcript import TranscriptResource
from .user import UserResource
from .userdevice import UserDeviceResource



from .resource import DialpadResource

class StatsExportResource(DialpadResource):
  """StatsExportResource implements python bindings for the Dialpad API's stats endpoints.
  See https://developers.dialpad.com/reference#stats for additional documentation.
  """
  _resource_path = ['stats']

  def post(self, coaching_group=False, days_ago_start=1, days_ago_end=30, is_today=False,
           export_type='stats', stat_type='calls', **kwargs):
    """Initiate a stats export.

    Args:
      coaching_group (bool, optional): Whether or not the the statistics should be for trainees of
                                       the coach with the given target_id.
      days_ago_start (int, optional): Start of the date range to get statistics for. This is the
                                      number of days to look back relative to the current day. Used
                                      in conjunction with days_ago_end to specify a range.
      days_ago_end (int, optional): End of the date range to get statistics for. This is the number
                                    of days to look back relative to the current day. Used in
                                    conjunction with days_ago_start to specify a range.
      is_today (bool, optional): Whether or not the statistics are for the current day.
                                 days_ago_start and days_ago_end are ignored if this is passed in
      export_type ("stats" or "records", optional): Whether to return aggregated statistics (stats),
                                                    or individual rows for each record (records).
      stat_type (str, optional): One of "calls", "texts", "voicemails", "recordings", "onduty",
                                 "csat", "dispositions". The type of statistics to be returned.
      office_id (int, optional): ID of the office to get statistics for. If a target_id and
                                 target_type are passed in this value is ignored and instead the
                                 target is used.
      target_id (int, optional): The ID of the target for which to return statistics.
      target_type (type, optional): One of "department", "office", "callcenter", "user", "room",
                                    "staffgroup", "callrouter", "channel", "coachinggroup",
                                    "unknown". The type corresponding to the target_id.
      timezone (str, optional): Timezone using a tz database name.

    See Also:
      https://developers.dialpad.com/reference#statsapi_processstats
    """

    data = {
      'coaching_group': coaching_group,
      'days_ago_start': str(days_ago_start),
      'days_ago_end': str(days_ago_end),
      'is_today': is_today,
      'export_type': export_type,
      'stat_type': stat_type,
    }

    data.update(kwargs)

    data = {k: v for k, v in data.iteritems() if v is not None}
    return self.request(method='POST', data=data)

  def get(self, export_id):
    """Retrieves the results of a stats export.

    Args:
      export_id (str, required): The export ID returned by the post method.

    See Also:
      https://developers.dialpad.com/reference#statsapi_getstats
    """
    return self.request([export_id])



from .resource import DialpadResource

class CallResource(DialpadResource):
  """CallResource implements python bindings for the Dialpad API's call endpoints.
  
  See https://developers.dialpad.com/reference#call for additional documentation.
  """
  _resource_path = ['call']

  def initiate_call(self, phone_number, user_id, **kwargs):
    """Initiates an oubound call to the specified phone number on behalf of the specified user.

    Note:
      This API will initiate the call by ringing the user's devices as well as ringing the specified
      number. When the user answers on their device, they will be connected with the call that is
      ringing the specified number.

      Optionally, group_type and group_id can be specified to cause the call to be routed through
      the specified group. This would be equivelant to the User initiating the call by selecting the
      specified group in the "New Call As" dropdown in the native app, or calling a contact that
      belongs to that group via the native app.

      In particular, the call will show up in that group's section of the app, and the external
      party will receive a call from the primary number of the specified group.

      Additionally, a specific device_id can be specified to cause that specific user-device to
      ring, rather than all of the user's devices.

    Args:
      phone_number (str, required): The e164-formatted number that should be called.
      user_id (int, required): The ID of the user that should be taking the call.
      group_id (int, optional): The ID of the call center, department, or office that should be used
                                to initiate the call.
      group_type (str, optional): One of "office", "department", or "callcenter", corresponding to
                                  the type of ID passed into group_type.
      device_id (str, optional): The ID of the specific user device that should ring.

    See Also:
      https://developers.dialpad.com/reference#callapi_call
    """
    return self.request(method='POST', data=dict(phone_number=phone_number, user_id=user_id,
                                                 **kwargs))



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.md') as f:
  readme = f.read()

setup(
  name='python-dialpad',
  version='1.1.2',
  description='A python wrapper for the Dialpad REST API',
  long_description=readme,
  long_description_content_type="text/markdown",
  author='Jake Nielsen',
  author_email='jnielsen@dialpad.com',
  license='MIT',
  url='https://github.com/dialpad/dialpad-python-sdk',
  install_requires=['cached-property', 'certifi', 'chardet', 'idna', 'requests', 'urllib3'],
  include_package_data=True,
  packages=find_packages()
)



#!/usr/bin/env python

"""Tests to automatically detect common issues with resource definitions.

In particular these tests will look through the files in dialpad-python-sdk/dialpad/resources/ and
ensure:

- All subclasses of DialpadResource are exposed directly in resources/__init__.py
- All resources are available as properties of DialpadClient
- Public methods defined on the concrete subclasses only make web requests that agree with
  the Dialpad API's open-api spec
"""

import inspect
import pkgutil
import pytest
import utils

from swagger_stub import swagger_stub

from dialpad.client import DialpadClient
from dialpad import resources
from dialpad.resources.resource import DialpadResource


# The "swagger_files_url" pytest fixture stubs out live requests with a schema validation check
# against the Dialpad API swagger spec.

# NOTE: Responses returned by the stub will not necessarily be a convincing dummy for the responses
#       returned by the live API, so some complex scenarios may not be possible to test using this
#       strategy.
@pytest.fixture(scope='module')
def swagger_files_url():
  return [
    (utils.resource_filepath('swagger_spec.json'), 'https://dialpad.com'),
  ]


class TestResourceSanity:
  """Sanity-tests for (largely) automatically validating new and existing client API methods.

  When new API resource methods are added to the library, examples of each method must be added to
  EX_METHOD_CALLS to allow the unit tests to call those methods and validate that the API requests
  they generate adhere to the swagger spec.

  The example calls should generally include as many keyword arguments as possible so that any
  potential mistakes in the parameter names, url path, and request body can be caught by the
  schema tests.

  Entries in the "EX_METHOD_CALLS" dictionary should be of the form:
  {
    '<ResourceClassName>': {
      'method_name': {
        'arg_name': arg_value,
        'other_arg_name': other_arg_value,
      },
      'other_method_name': etc... 
    }
  }
  """

  EX_METHOD_CALLS = {
    'BlockedNumberResource': {
      'list': {},
      'block_numbers': {
        'numbers': ['+12223334444']
      },
      'unblock_numbers': {
        'numbers': ['+12223334444']
      },
      'get': {
        'number': '+12223334444'
      },
    },
    'CallResource': {
      'initiate_call': {
        'phone_number': '+12223334444',
        'user_id': '123',
        'group_id': '123',
        'group_type': 'department',
        'device_id': '123',
      },
    },
    'CallRouterResource': {
      'get': {
        'call_router_id': '123',
      },
      'post': {
        'name': 'Test Router',
        'routing_url': 'fakeurl.com/url',
        'office_id': '123',
        'default_target_id': '123',
        'default_target_type': 'user',
        'enabled': True,
        'secret': '123',
      },
      'patch': {
        'call_router_id': '123',
        'name': 'Test Router',
        'routing_url': 'fakeurl.com/url',
        'office_id': '123',
        'default_target_id': '123',
        'default_target_type': 'user',
        'enabled': True,
        'secret': '123',
      },
      'delete': {
        'call_router_id': '123',
      },
      'assign_number': {
        'call_router_id': '123',
        'area_code': '519',
      },
    },
    'CallbackResource': {
      'enqueue_callback': {
        'call_center_id': '123',
        'phone_number': '+12223334444',
      },
    },
    'CallCenterResource': {
      'get': {
        'call_center_id': '123',
      },
      'get_operators': {
        'call_center_id': '123',
      },
      'add_operator': {
        'call_center_id': '123',
        'user_id': '123',
        'skill_level': '10',
        'role': 'supervisor',
        'license_type': 'lite_support_agents',
        'keep_paid_numbers': False,
      },
      'remove_operator': {
        'call_center_id': '123',
        'user_id': '123',
      },
    },
    'CompanyResource': {
      'get': {},
    },
    'ContactResource': {
      'list': {
        'owner_id': '123',
      },
      'create': {
        'first_name': 'Testiel',
        'last_name': 'McTestersen',
        'company_name': 'ABC',
        'emails': ['tmtesten@test.com'],
        'extension': '123',
        'job_title': 'Eric the half-a-bee',
        'owner_id': '123',
        'phones': ['+12223334444'],
        'trunk_group': '123',
        'urls': ['test.com/about'],
      },
      'create_with_uid': {
        'first_name': 'Testiel',
        'last_name': 'McTestersen',
        'uid': 'UUID-updownupdownleftrightab',
        'company_name': 'ABC',
        'emails': ['tmtesten@test.com'],
        'extension': '123',
        'job_title': 'Eric the half-a-bee',
        'phones': ['+12223334444'],
        'trunk_group': '123',
        'urls': ['test.com/about'],
      },
      'delete': {
        'contact_id': '123',
      },
      'get': {
        'contact_id': '123',
      },
      'patch': {
        'contact_id': '123',
        'first_name': 'Testiel',
        'last_name': 'McTestersen',
        'company_name': 'ABC',
        'emails': ['tmtesten@test.com'],
        'extension': '123',
        'job_title': 'Eric the half-a-bee',
        'phones': ['+12223334444'],
        'trunk_group': '123',
        'urls': ['test.com/about'],
      },
    },
    'DepartmentResource': {
      'get': {
        'department_id': '123',
      },
      'get_operators': {
        'department_id': '123',
      },
      'add_operator': {
        'department_id': '123',
        'operator_id': '123',
        'operator_type': 'room',
        'role': 'operator',
      },
      'remove_operator': {
        'department_id': '123',
        'operator_id': '123',
        'operator_type': 'room',
      },
    },
    'EventSubscriptionResource': {
      'list_call_event_subscriptions': {
        'target_id': '123',
        'target_type': 'room',
      },
      'get_call_event_subscription': {
        'subscription_id': '123',
      },
      'put_call_event_subscription': {
        'subscription_id': '123',
        'url': 'test.com/subhook',
        'secret': 'badsecret',
        'enabled': True,
        'group_calls_only': False,
        'target_id': '123',
        'target_type': 'office',
        'call_states': ['connected', 'queued'],
      },
      'delete_call_event_subscription': {
        'subscription_id': '123',
      },
      'list_sms_event_subscriptions': {
        'target_id': '123',
        'target_type': 'room',
      },
      'get_sms_event_subscription': {
        'subscription_id': '123',
      },
      'put_sms_event_subscription': {
        'subscription_id': '123',
        'url': 'test.com/subhook',
        'secret': 'badsecret',
        'direction': 'outbound',
        'enabled': True,
        'target_id': '123',
        'target_type': 'office',
      },
      'delete_sms_event_subscription': {
        'subscription_id': '123',
      },
    },
    'NumberResource': {
      'list': {
        'status': 'available',
      },
      'get': {
        'number': '+12223334444',
      },
      'unassign': {
        'number': '+12223334444',
      },
      'assign': {
        'number': '+12223334444',
        'target_id': '123',
        'target_type': 'office',
      },
    },
    'OfficeResource': {
      'list': {},
      'get': {
        'office_id': '123',
      },
      'assign_number': {
        'office_id': '123',
        'number': '+12223334444',
      },
      'get_operators': {
        'office_id': '123',
      },
      'unassign_number': {
        'office_id': '123',
        'number': '+12223334444',
      },
      'get_call_centers': {
        'office_id': '123',
      },
      'get_departments': {
        'office_id': '123',
      },
      'get_plan': {
        'office_id': '123',
      },
      'update_licenses': {
        'office_id': '123',
        'fax_line_delta': '2',
      },
    },
    'RoomResource': {
      'list': {
        'office_id': '123',
      },
      'create': {
        'name': 'Where it happened',
        'office_id': '123',
      },
      'generate_international_pin': {
        'customer_ref': 'Burr, sir',
      },
      'delete': {
        'room_id': '123',
      },
      'get': {
        'room_id': '123',
      },
      'update': {
        'room_id': '123',
        'name': 'For the last tiiiime',
        'phone_numbers': ['+12223334444'],
      },
      'assign_number': {
        'room_id': '123',
        'number': '+12223334444',
      },
      'unassign_number': {
        'room_id': '123',
        'number': '+12223334444',
      },
      'get_deskphones': {
        'room_id': '123',
      },
      'create_deskphone': {
        'room_id': '123',
        'mac_address': 'Tim Cook',
        'name': 'The red one.',
        'phone_type': 'polycom',
      },
      'delete_deskphone': {
        'room_id': '123',
        'deskphone_id': '123',
      },
      'get_deskphone': {
        'room_id': '123',
        'deskphone_id': '123',
      },
    },
    'SMSResource': {
      'send_sms': {
        'user_id': '123',
        'to_numbers': ['+12223334444'],
        'text': 'Itemized list to follow.',
        'infer_country_code': False,
        'sender_group_id': '123',
        'sender_group_type': 'callcenter',
      },
    },
    'StatsExportResource': {
      'post': {
        'coaching_group': False,
        'days_ago_start': '1',
        'days_ago_end': '2',
        'is_today': False,
        'export_type': 'records',
        'stat_type': 'calls',
        'office_id': '123',
        'target_id': '123',
        'target_type': 'callcenter',
        'timezone': 'America/New_York',
      },
      'get': {
        'export_id': '123',
      },
    },
    'TranscriptResource': {
      'get': {
        'call_id': '123',
      },
    },
    'UserResource': {
      'list': {
        'email': 'tmtesten@test.com',
        'state': 'suspended',
      },
      'create': {
        'email': 'tmtesten@test.com',
        'office_id': '123',
        'first_name': 'Testietta',
        'last_name': 'McTestersen',
        'license': 'lite_support_agents',
      },
      'delete': {
        'user_id': '123',
      },
      'get': {
        'user_id': '123',
      },
      'update': {
        'user_id': '123',
        'admin_office_ids': ['123'],
        'emails': ['tmtesten@test.com'],
        'extension': '123',
        'first_name': 'Testietta',
        'last_name': 'McTestersen',
        'forwarding_numbers': ['+12223334444'],
        'is_super_admin': True,
        'job_title': 'Administraterar',
        'license': 'lite_lines',
        'office_id': '123',
        'phone_numbers': ['+12223334444'],
        'state': 'active',
      },
      'toggle_call_recording': {
        'user_id': '123',
        'is_recording': False,
        'play_message': True,
        'recording_type': 'group',
      },
      'assign_number': {
        'user_id': '123',
        'number': '+12223334444',
      },
      'initiate_call': {
        'user_id': '123',
        'phone_number': '+12223334444',
        'custom_data': 'Y u call self?',
        'group_id': '123',
        'group_type': 'department',
        'outbound_caller_id': 'O.0',
      },
      'unassign_number': {
        'user_id': '123',
        'number': '+12223334444',
      },
      'get_deskphones': {
        'user_id': '123',
      },
      'create_deskphone': {
        'user_id': '123',
        'mac_address': 'Tim Cook',
        'name': 'The red one.',
        'phone_type': 'polycom',
      },
      'delete_deskphone': {
        'user_id': '123',
        'deskphone_id': '123',
      },
      'get_deskphone': {
        'user_id': '123',
        'deskphone_id': '123',
      },
    },
    'UserDeviceResource': {
      'get': {
        'device_id': '123',
      },
      'list': {
        'user_id': '123',
      },
    },
  }

  def get_method_example_kwargs(self, resource_instance, resource_method):
    """Returns the appropriate kwargs to use when sanity-checking API resource methods."""
    class_msg = 'DialpadResource subclass "%s" must have an entry in EX_METHOD_CALLS'

    class_name = resource_instance.__class__.__name__
    assert class_name in self.EX_METHOD_CALLS, class_msg % class_name

    method_msg = 'Method "%s.%s" must have an entry in EX_METHOD_CALLS'
    method_name = resource_method.__name__
    assert method_name in self.EX_METHOD_CALLS[class_name], method_msg % (class_name, method_name)

    return self.EX_METHOD_CALLS[class_name][method_name]

  def _get_resource_submodule_names(self):
    """Returns an iterator of python modules that exist in the dialpad/resources directory."""
    for importer, modname, ispkg in pkgutil.iter_modules(resources.__path__):
      if modname == 'resource':
        continue

      if ispkg:
        continue

      yield modname

  def _get_resource_submodules(self):
    """Returns an iterator of python modules that are exposed via from dialpad.resources import *"""
    for modname in self._get_resource_submodule_names():
      if hasattr(resources, modname):
        yield getattr(resources, modname)

  def _get_resource_classes(self):
    """Returns an iterator of DialpadResource subclasses that are exposed under dialpad.resources"""
    for mod in self._get_resource_submodules():
      for k, v in mod.__dict__.iteritems():
        if not inspect.isclass(v):
          continue

        if not issubclass(v, DialpadResource):
          continue

        if v == DialpadResource:
          continue

        yield v

  def test_resources_properly_imported(self):
    """Verifies that all modules definied in the resources directory are properly exposed under
    dialpad.resources.
    """
    exposed_resources = dir(resources)

    msg = '"%s" module is present in the resources directory, but is not imported in ' \
          'resources/__init__.py'

    for modname in self._get_resource_submodule_names():
      assert modname in exposed_resources, msg % modname

  def test_resource_classes_properly_exposed(self):
    """Verifies that all subclasses of DialpadResource that are defined in the resources directory
    are also exposed as direct members of the resources module.
    """
    exposed_resources = dir(resources)

    msg = '"%(name)s" resource class is present in the resources package, but is not exposed ' \
          'directly as resources.%(name)s via resources/__init__.py'

    for c in self._get_resource_classes():
      assert c.__name__ in exposed_resources, msg % {'name': c.__name__}

  def test_request_conformance(self, swagger_stub):
    """Verifies that all API requests produced by this library conform to the swagger spec.

    Although this test cannot guarantee that the requests are semantically correct, it can at least
    determine whether they are schematically correct.

    This test will also fail if there are no test-kwargs defined in EX_METHOD_CALLS for any public
    method implemented by a subclass of DialpadResource.
    """

    # Construct a DialpadClient with a fake API key.
    dp = DialpadClient('123')

    # Iterate through the attributes on the client object to find the API resource accessors.
    for a in dir(dp):
      resource_instance = getattr(dp, a)

      # Skip any attributes that are not DialpadResources
      if not isinstance(resource_instance, DialpadResource):
        continue

      print ''
      print 'Verifying request format of %s methods' % resource_instance.__class__.__name__

      # Iterate through the attributes on the resource instance.
      for method_attr in dir(resource_instance):
        # Skip private attributes.
        if method_attr.startswith('_'):
          continue

        # Skip attributes that are not unique to this particular subclass of DialpadResource.
        if hasattr(DialpadResource, method_attr):
          continue

        # Skip attributes that are not functions.
        resource_method = getattr(resource_instance, method_attr)
        if not callable(resource_method):
          continue

        # Skip attributes that are not instance methods.
        arg_names = inspect.getargspec(resource_method).args
        if not arg_names or arg_names[0] != 'self':
          continue

        # Fetch example kwargs to test the method (and raise if they haven't been provided).
        method_kwargs = self.get_method_example_kwargs(resource_instance, resource_method)

        # Call the method, and allow the swagger mock to raise an exception if it encounters a
        # schema error.
        print 'Testing %s with kwargs: %s' % (method_attr, method_kwargs)
        resource_method(**method_kwargs)



from .resource import DialpadResource

class UserDeviceResource(DialpadResource):
  """UserDeviceResource implements python bindings for the Dialpad API's userdevice endpoints.
  See https://developers.dialpad.com/reference#userdevices for additional documentation.
  """
  _resource_path = ['userdevices']

  def get(self, device_id):
    """Gets a user device by ID.

    Args:
      device_id (str, required): The ID of the device.

    See Also:
      https://developers.dialpad.com/reference#userdeviceapi_getdevice
    """
    return self.request([device_id])

  def list(self, user_id, limit=25, **kwargs):
    """Lists the devices for a specific user.

    Args:
      user_id (int, required): The ID of the user.
      limit (int, optional): the number of devices to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#userdeviceapi_listuserdevices
    """
    return self.request(data=dict(user_id=user_id, limit=limit, **kwargs))



from .resource import DialpadResource

class UserResource(DialpadResource):
  """UserResource implements python bindings for the Dialpad API's user endpoints.
  See https://developers.dialpad.com/reference#users for additional documentation.
  """
  _resource_path = ['users']

  def list(self, limit=25, **kwargs):
    """Lists users in the company.

    Args:
      email (str, optional): Limits results to users with a matching email address.
      state (str, optional): Limits results to users in the specified state (one of "active",
                             "canceled", "suspended", "pending", "deleted", "all)
      limit (int, optional): The number of users to fetch per request.

    See Also:
      https://developers.dialpad.com/reference#userapi_listusers
    """
    return self.request(data=dict(limit=limit, **kwargs))

  def create(self, email, office_id, **kwargs):
    """Creates a new user.

    Args:
      email (str, required): The user's email address.
      office_id (int, required): The ID of the office that the user should belong to.
      first_name (str, optional): The first name of the user.
      last_name (str, optional): The last name of the user.
      license (str, optional): The license that the user should be created with. (One of "talk",
                               "agents", "lite_support_agents", "lite_lines")

    See Also:
      https://developers.dialpad.com/reference#userapi_createuser
    """
    return self.request(method='POST', data=dict(email=email, office_id=office_id, **kwargs))

  def delete(self, user_id):
    """Deletes a user.

    Args:
      user_id (int, required): The ID of the user to delete.

    See Also:
      https://developers.dialpad.com/reference#userapi_deleteuser
    """
    return self.request([user_id], method='DELETE')

  def get(self, user_id):
    """Gets a user by ID.

    Args:
      user_id (int, required): The ID of the user.

    See Also:
      https://developers.dialpad.com/reference#userapi_getuser
    """
    return self.request([user_id])

  def update(self, user_id, **kwargs):
    """Updates a user by ID.

    Note:
      The "phone_numbers" argument can be used to re-order or unassign numbers, but it cannot be
      used to assign new numbers. To assign new numbers to a user, please use the number assignment
      API instead.

    Args:
      user_id (int, required): The ID of the user.
      admin_office_ids (list<str>, optional): The office IDs that this user should be an admin of.
      emails (list<str>, optional): The email addresses that should be assoiciated with this user.
      extension (str, optional): The extension that this user can be reached at.
      first_name (str, optional): The first name of the user.
      last_name (str, optional): The last name of the user.
      forwarding_numbers (list<str>, optional): The e164-formatted numbers that should also ring
                                                when the user receives a Dialpad call.
      is_super_admin (bool, optional): Whether this user should be a company-level admin.
      job_title (str, optional): The user's job title.
      license (str, optional): The user's license type. Changing this affects billing for the user.
      office_id (int, optional): The ID of office to which this user should belong.
      phone_numbers (list<str>, optional): The e164-formatted numbers that should be assigned to
                                           this user.
      state (str, optional): The state of the user (One of "suspended", "active")

    See Also:
      https://developers.dialpad.com/reference#userapi_updateuser
    """
    return self.request([user_id], method='PATCH', data=kwargs)

  def toggle_call_recording(self, user_id, **kwargs):
    """Turn call recording on or off for a user's active call.

    Args:
      user_id (int, required): The ID of the user.
      is_recording (bool, optional): Whether recording should be turned on.
      play_message (bool, optional): Whether a message should be played to the user to notify them
                                     that they are now being (or no longer being) recorded.
      recording_type (str, optional): One of "user", "group", or "all". If set to "user", then only
                                      the user's individual calls will be recorded. If set to
                                      "group", then only calls in which the user is acting as an
                                      operator will be recorded. If set to "all", then all of the
                                      user's calls will be recorded.

    See Also:
      https://developers.dialpad.com/reference#callapi_updateactivecall
    """
    return self.request([user_id, 'activecall'], method='PATCH', data=kwargs)

  def assign_number(self, user_id, **kwargs):
    """Assigns a new number to the user.

    Args:
      user_id (int, required): The ID of the user to which the number should be assigned.
      number (str, optional): An e164 number that has already been allocated to the company's
                              reserved number pool that should be re-assigned to this user.
      area_code (str, optional): The area code to use to filter the set of available numbers to be
                                 assigned to this user.

    See Also:
      https://developers.dialpad.com/reference#numberapi_assignnumbertouser
    """
    return self.request([user_id, 'assign_number'], method='POST', data=kwargs)

  def initiate_call(self, user_id, phone_number, **kwargs):
    """Causes a user's native Dialpad application to initiate an outbound call.

    Args:
      user_id (int, required): The ID of the user.
      phone_number (str, required): The e164-formatted number to call.
      custom_data (str, optional): free-form extra data to associate with the call.
      group_id (str, optional): The ID of a group that will be used to initiate the call.
      group_type (str, optional): The type of a group that will be used to initiate the call.
      outbound_caller_id (str, optional): The e164-formatted number shown to the call recipient
                                  (or "blocked"). If set to "blocked", the recipient will receive a
                                  call from "unknown caller".

    See Also:
      https://developers.dialpad.com/reference#callapi_initiatecall
    """
    data = {
      'phone_number': phone_number
    }
    for k in ['group_id', 'group_type', 'outbound_caller_id', 'custom_data']:
      if k in kwargs:
        data[k] = kwargs.pop(k)
    assert not kwargs
    return self.request([user_id, 'initiate_call'], method='POST', data=data)

  def unassign_number(self, user_id, number):
    """Unassigns the specified number from the specified user.

    Args:
      user_id (int, required): The ID of the user.
      number (str, required): The e164-formatted number that should be unassigned from the user.

    See Also:
      https://developers.dialpad.com/reference#numberapi_unassignnumberfromuser
    """
    return self.request([user_id, 'unassign_number'], method='POST', data={'number': number})

  def get_deskphones(self, user_id):
    """Lists the desk phones that are associated with a user.

    Args:
      user_id (int, required): The ID of the user.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_listuserdeskphones
    """
    return self.request([user_id, 'deskphones'], method='GET')

  def create_deskphone(self, user_id, mac_address, name, phone_type):
    """Creates a desk phone belonging to the specified user.

    Args:
      user_id (int, required): The ID of the user.
      mac_address (str, required): MAC address of the desk phone.
      name (str, required): A human-readable name for the desk phone.
      phone_type (str, required): Type (vendor) of the desk phone. One of "obi", "polycom", "sip",
                                  "mini", "audiocodes", "yealink". Use "sip" for generic types.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_createuserdeskphone
    """
    return self.request([user_id, 'deskphones'], method='POST', data={
      'mac_address': mac_address,
      'name': name,
      'type': phone_type,
    })

  def delete_deskphone(self, user_id, deskphone_id):
    """Deletes the specified desk phone.

    Args:
      user_id (int, required): The ID of the user.
      deskphone_id (str, required): The ID of the desk phone.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_deleteuserdeskphone
    """
    return self.request([user_id, 'deskphones', deskphone_id], method='DELETE')

  def get_deskphone(self, user_id, deskphone_id):
    """Gets the specified desk phone.

    Args:
      user_id (int, required): The ID of the user.
      deskphone_id (str, required): The ID of the desk phone.

    See Also:
      https://developers.dialpad.com/reference#deskphoneapi_getuserdeskphone
    """
    return self.request([user_id, 'deskphones', deskphone_id], method='GET')



from .resource import DialpadResource

class NumberResource(DialpadResource):
  """NumberResource implements python bindings for the Dialpad API's number endpoints.
  
  See https://developers.dialpad.com/reference#numbers for additional documentation.
  """
  _resource_path = ['numbers']

  def list(self, limit=25, **kwargs):
    """List all phone numbers in the company.

    Args:
      limit (int, optional): The number of numbers to fetch per request
      status (str, optional): If provided, the results will only contain numbers with the specified
                              status. Must be one of: "available", "pending", "office",
                              "department", "call_center", "user", "room", "porting", "call_router",
                              or "dynamic_caller"

    See Also:
      https://developers.dialpad.com/reference#numberapi_listnumbers
    """
    return self.request(method='GET', data=dict(limit=limit, **kwargs))

  def get(self, number):
    """Gets a specific number object.

    Args:
      number (str, required): An e164-formatted number.

    See Also:
      https://developers.dialpad.com/reference#numberapi_getnumber
    """
    return self.request([number], method='GET')

  def unassign(self, number, release=False):
    """Unassigns the specified number.

    Args:
      number (str, required): An e164-formatted number.
      release (bool, optional): If the "release" flag is omitted or set to False, the number will
                                be returned to the company pool (i.e. your company will still own
                                the number, but it will no longer be assigned to any targets).
                                If the "release" flag is set, then the number will be beamed back
                                to the Dialpad mothership.

    See Also:
      https://developers.dialpad.com/reference#numberapi_unassignnumber
    """
    return self.request([number], method='DELETE', data={'release': release})

  def assign(self, number, target_id, target_type, primary=True):
    """Assigns the specified number to the specified target.

    Args:
      number (str, required): The e164-formatted number that should be assigned.
      target_id (int, required): The ID of the target to which the number should be assigned.
      target_type (str, required): The type corresponding to the provided target ID.
      primary (bool, optional): (Defaults to True) If the "primary" flag is set, then the assigned
                                number will become the primary number of the specified target.

    See Also:
      https://developers.dialpad.com/reference#numberapi_assignnumber
    """
    return self.request([number, 'assign'], method='POST', data={
        'target_id': target_id,
        'target_type': target_type,
        'primary': primary
    })
